import codecs

from .. import codec, incremental

ANSEL_TO_UNICODE = {
    b"\x00": u"\u0000",  # NULL CHARACTER
    b"\x01": u"\u0001",  # START OF HEADING
    b"\x02": u"\u0002",  # START OF TEXT
    b"\x03": u"\u0003",  # END-OF-TEXT CHARACTER
    b"\x04": u"\u0004",  # END-OF-TRANSMISSION CHARACTER
    b"\x05": u"\u0005",  # ENQUIRY CHARACTER
    b"\x06": u"\u0006",  # ACKNOWLEDGE CHARACTER
    b"\x07": u"\u0007",  # BELL CHARACTER
    b"\x08": u"\u0008",  # BACKSPACE
    b"\x09": u"\u0009",  # HORIZONTAL TAB
    b"\x0A": u"\u000A",  # LINE FEED
    b"\x0B": u"\u000B",  # VERTICAL TAB
    b"\x0C": u"\u000C",  # FORM FEED
    b"\x0D": u"\u000D",  # CARRIAGE RETURN
    b"\x0E": u"\u000E",  # SHIFT OUT
    b"\x0F": u"\u000F",  # SHIFT IN
    b"\x10": u"\u0010",  # DATA LINK ESCAPE
    b"\x11": u"\u0011",  # DEVICE CONTROL 1
    b"\x12": u"\u0012",  # DEVICE CONTROL 2
    b"\x13": u"\u0013",  # DEVICE CONTROL 3
    b"\x14": u"\u0014",  # DEVICE CONTROL 4
    b"\x15": u"\u0015",  # NEGATIVE-ACKNOWLEDGE CHARACTER
    b"\x16": u"\u0016",  # SYNCHRONOUS IDLE
    b"\x17": u"\u0017",  # END OF TRANSMISSION BLOCK
    b"\x18": u"\u0018",  # CANCEL CHARACTER
    b"\x19": u"\u0019",  # END OF MEDIUM
    b"\x1A": u"\u001A",  # SUBSTITUTE CHARACTER
    b"\x1B": u"\u001B",  # ESCAPE CHARACTER
    b"\x1C": u"\u001C",  # FILE SEPARATOR
    b"\x1D": u"\u001D",  # GROUP SEPARATOR
    b"\x1E": u"\u001E",  # RECORD SEPARATOR
    b"\x1F": u"\u001F",  # UNIT SEPARATOR
    b"\x20": u"\u0020",  # SPACE
    b"\x21": u"\u0021",  # EXCLAMATION MARK
    b"\x22": u"\u0022",  # QUOTATION MARK
    b"\x23": u"\u0023",  # NUMBER SIGN
    b"\x24": u"\u0024",  # DOLLAR SIGN
    b"\x25": u"\u0025",  # PERCENT SIGN
    b"\x26": u"\u0026",  # AMPERSAND
    b"\x27": u"\u0027",  # APOSTROPHE
    b"\x28": u"\u0028",  # LEFT PARENTHESIS
    b"\x29": u"\u0029",  # RIGHT PARENTHESIS
    b"\x2A": u"\u002A",  # ASTERISK
    b"\x2B": u"\u002B",  # PLUS SIGN
    b"\x2C": u"\u002C",  # COMMA
    b"\x2D": u"\u002D",  # HYPHEN-MINUS
    b"\x2E": u"\u002E",  # FULL STOP
    b"\x2F": u"\u002F",  # SOLIDUS
    b"\x30": u"\u0030",  # DIGIT ZERO
    b"\x31": u"\u0031",  # DIGIT ONE
    b"\x32": u"\u0032",  # DIGIT TWO
    b"\x33": u"\u0033",  # DIGIT THREE
    b"\x34": u"\u0034",  # DIGIT FOUR
    b"\x35": u"\u0035",  # DIGIT FIVE
    b"\x36": u"\u0036",  # DIGIT SIX
    b"\x37": u"\u0037",  # DIGIT SEVEN
    b"\x38": u"\u0038",  # DIGIT EIGHT
    b"\x39": u"\u0039",  # DIGIT NINE
    b"\x3A": u"\u003A",  # COLON
    b"\x3B": u"\u003B",  # SEMICOLON
    b"\x3C": u"\u003C",  # LESS-THAN SIGN
    b"\x3D": u"\u003D",  # EQUALS SIGN
    b"\x3E": u"\u003E",  # GREATER-THAN SIGN
    b"\x3F": u"\u003F",  # QUESTION MARK
    b"\x40": u"\u0040",  # COMMERCIAL AT
    b"\x41": u"\u0041",  # LATIN CAPITAL LETTER A
    b"\x42": u"\u0042",  # LATIN CAPITAL LETTER B
    b"\x43": u"\u0043",  # LATIN CAPITAL LETTER C
    b"\x44": u"\u0044",  # LATIN CAPITAL LETTER D
    b"\x45": u"\u0045",  # LATIN CAPITAL LETTER E
    b"\x46": u"\u0046",  # LATIN CAPITAL LETTER F
    b"\x47": u"\u0047",  # LATIN CAPITAL LETTER G
    b"\x48": u"\u0048",  # LATIN CAPITAL LETTER H
    b"\x49": u"\u0049",  # LATIN CAPITAL LETTER I
    b"\x4A": u"\u004A",  # LATIN CAPITAL LETTER J
    b"\x4B": u"\u004B",  # LATIN CAPITAL LETTER K
    b"\x4C": u"\u004C",  # LATIN CAPITAL LETTER L
    b"\x4D": u"\u004D",  # LATIN CAPITAL LETTER M
    b"\x4E": u"\u004E",  # LATIN CAPITAL LETTER N
    b"\x4F": u"\u004F",  # LATIN CAPITAL LETTER O
    b"\x50": u"\u0050",  # LATIN CAPITAL LETTER P
    b"\x51": u"\u0051",  # LATIN CAPITAL LETTER Q
    b"\x52": u"\u0052",  # LATIN CAPITAL LETTER R
    b"\x53": u"\u0053",  # LATIN CAPITAL LETTER S
    b"\x54": u"\u0054",  # LATIN CAPITAL LETTER T
    b"\x55": u"\u0055",  # LATIN CAPITAL LETTER U
    b"\x56": u"\u0056",  # LATIN CAPITAL LETTER V
    b"\x57": u"\u0057",  # LATIN CAPITAL LETTER W
    b"\x58": u"\u0058",  # LATIN CAPITAL LETTER X
    b"\x59": u"\u0059",  # LATIN CAPITAL LETTER Y
    b"\x5A": u"\u005A",  # LATIN CAPITAL LETTER Z
    b"\x5B": u"\u005B",  # LEFT SQUARE BRACKET
    b"\x5C": u"\u005C",  # REVERSE SOLIDUS
    b"\x5D": u"\u005D",  # RIGHT SQUARE BRACKET
    b"\x5E": u"\u005E",  # CIRCUMFLEX ACCENT
    b"\x5F": u"\u005F",  # LOW LINE
    b"\x60": u"\u0060",  # GRAVE ACCENT
    b"\x61": u"\u0061",  # LATIN SMALL LETTER A
    b"\x62": u"\u0062",  # LATIN SMALL LETTER B
    b"\x63": u"\u0063",  # LATIN SMALL LETTER C
    b"\x64": u"\u0064",  # LATIN SMALL LETTER D
    b"\x65": u"\u0065",  # LATIN SMALL LETTER E
    b"\x66": u"\u0066",  # LATIN SMALL LETTER F
    b"\x67": u"\u0067",  # LATIN SMALL LETTER G
    b"\x68": u"\u0068",  # LATIN SMALL LETTER H
    b"\x69": u"\u0069",  # LATIN SMALL LETTER I
    b"\x6A": u"\u006A",  # LATIN SMALL LETTER J
    b"\x6B": u"\u006B",  # LATIN SMALL LETTER K
    b"\x6C": u"\u006C",  # LATIN SMALL LETTER L
    b"\x6D": u"\u006D",  # LATIN SMALL LETTER M
    b"\x6E": u"\u006E",  # LATIN SMALL LETTER N
    b"\x6F": u"\u006F",  # LATIN SMALL LETTER O
    b"\x70": u"\u0070",  # LATIN SMALL LETTER P
    b"\x71": u"\u0071",  # LATIN SMALL LETTER Q
    b"\x72": u"\u0072",  # LATIN SMALL LETTER R
    b"\x73": u"\u0073",  # LATIN SMALL LETTER S
    b"\x74": u"\u0074",  # LATIN SMALL LETTER T
    b"\x75": u"\u0075",  # LATIN SMALL LETTER U
    b"\x76": u"\u0076",  # LATIN SMALL LETTER V
    b"\x77": u"\u0077",  # LATIN SMALL LETTER W
    b"\x78": u"\u0078",  # LATIN SMALL LETTER X
    b"\x79": u"\u0079",  # LATIN SMALL LETTER Y
    b"\x7A": u"\u007A",  # LATIN SMALL LETTER Z
    b"\x7B": u"\u007B",  # LEFT CURLY BRACKET
    b"\x7C": u"\u007C",  # VERTICAL LINE
    b"\x7D": u"\u007D",  # RIGHT CURLY BRACKET
    b"\x7E": u"\u007E",  # TILDE
    b"\x7F": u"\u007F",  # DELETE
    b"\xA1": u"\u0141",  # LATIN CAPITAL LETTER L WITH STROKE
    b"\xA2": u"\u00D8",  # LATIN CAPITAL LETTER O WITH STROKE
    b"\xA3": u"\u0110",  # LATIN CAPITAL LETTER D WITH STROKE
    b"\xA4": u"\u00DE",  # LATIN CAPITAL LETTER THORN
    b"\xA5": u"\u00C6",  # LATIN CAPITAL LETTER AE
    b"\xA6": u"\u0152",  # LATIN CAPITAL LIGATURE OE
    b"\xA7": u"\u02B9",  # MODIFIER LETTER PRIME
    b"\xA8": u"\u00B7",  # MIDDLE DOT
    b"\xA9": u"\u266D",  # MUSIC FLAT SIGN
    b"\xAA": u"\u00AE",  # REGISTERED SIGN
    b"\xAB": u"\u00B1",  # PLUS-MINUS SIGN
    b"\xAC": u"\u01A0",  # LATIN CAPITAL LETTER O WITH HORN
    b"\xAD": u"\u01AF",  # LATIN CAPITAL LETTER U WITH HORN
    b"\xAE": u"\u02BC",  # MODIFIER LETTER APOSTROPHE
    b"\xB0": u"\u02BB",  # MODIFIER LETTER TURNED COMMA
    b"\xB1": u"\u0142",  # LATIN SMALL LETTER L WITH STROKE
    b"\xB2": u"\u00F8",  # LATIN SMALL LETTER O WITH STROKE
    b"\xB3": u"\u0111",  # LATIN SMALL LETTER D WITH STROKE
    b"\xB4": u"\u00FE",  # LATIN SMALL LETTER THORN
    b"\xB5": u"\u00E6",  # LATIN SMALL LETTER AE
    b"\xB6": u"\u0153",  # LATIN SMALL LIGATURE OE
    b"\xB7": u"\u02BA",  # MODIFIER LETTER DOUBLE PRIME
    b"\xB8": u"\u0131",  # LATIN SMALL LETTER DOTLESS I
    b"\xB9": u"\u00A3",  # POUND SIGN
    b"\xBA": u"\u00F0",  # LATIN SMALL LETTER ETH
    b"\xBC": u"\u01A1",  # LATIN SMALL LETTER O WITH HORN
    b"\xBD": u"\u01B0",  # LATIN SMALL LETTER U WITH HORN
    b"\xC0": u"\u00B0",  # DEGREE SIGN
    b"\xC1": u"\u2113",  # SCRIPT SMALL L
    b"\xC2": u"\u2117",  # SOUND RECORDING COPYRIGHT
    b"\xC3": u"\u00A9",  # COPYRIGHT SIGN
    b"\xC4": u"\u266F",  # MUSIC SHARP SIGN
    b"\xC5": u"\u00BF",  # INVERTED QUESTION MARK
    b"\xC6": u"\u00A1",  # INVERTED EXCLAMATION MARK
}

ANSEL_TO_UNICODE_MODIFIERS = {
    b"\xE0": u"\u0309",  # COMBINING HOOK ABOVE
    b"\xE1": u"\u0300",  # COMBINING GRAVE ACCENT
    b"\xE2": u"\u0301",  # COMBINING ACUTE ACCENT
    b"\xE3": u"\u0302",  # COMBINING CIRCUMFLEX ACCENT
    b"\xE4": u"\u0303",  # COMBINING TILDE
    b"\xE5": u"\u0304",  # COMBINING MACRON
    b"\xE6": u"\u0306",  # COMBINING BREVE
    b"\xE7": u"\u0307",  # COMBINING DOT ABOVE
    b"\xE8": u"\u0308",  # COMBINING DIAERESIS
    b"\xE9": u"\u030C",  # COMBINING CARON
    b"\xEA": u"\u030A",  # COMBINING RING ABOVE
    b"\xEB": u"\uFE20",  # COMBINING LIGATURE LEFT HALF
    b"\xEC": u"\uFE21",  # COMBINING LIGATURE RIGHT HALF
    b"\xED": u"\u0315",  # COMBINING COMMA ABOVE RIGHT
    b"\xEE": u"\u030B",  # COMBINING DOUBLE ACUTE ACCENT
    b"\xEF": u"\u0310",  # COMBINING CANDRABINDU
    b"\xF0": u"\u0327",  # COMBINING CEDILLA
    b"\xF1": u"\u0328",  # COMBINING OGONEK
    b"\xF2": u"\u0323",  # COMBINING DOT BELOW
    b"\xF3": u"\u0324",  # COMBINING DIAERESIS BELOW
    b"\xF4": u"\u0325",  # COMBINING RING BELOW
    b"\xF5": u"\u0333",  # COMBINING DOUBLE LOW LINE
    b"\xF6": u"\u0332",  # COMBINING LOW LINE
    b"\xF7": u"\u0326",  # COMBINING COMMA BELOW
    b"\xF8": u"\u031C",  # COMBINING LEFT HALF RING BELOW
    b"\xF9": u"\u032E",  # COMBINING BREVE BELOW
    b"\xFA": u"\uFE22",  # COMBINING DOUBLE TILDE LEFT HALF
    b"\xFB": u"\uFE23",  # COMBINING DOUBLE TILDE RIGHT HALF
    b"\xFE": u"\u0313",  # COMBINING COMMA ABOVE
}

UNICODE_TO_ANSEL = {
    u"\u0000": b"\x00",  # NULL CHARACTER
    u"\u0001": b"\x01",  # START OF HEADING
    u"\u0002": b"\x02",  # START OF TEXT
    u"\u0003": b"\x03",  # END-OF-TEXT CHARACTER
    u"\u0004": b"\x04",  # END-OF-TRANSMISSION CHARACTER
    u"\u0005": b"\x05",  # ENQUIRY CHARACTER
    u"\u0006": b"\x06",  # ACKNOWLEDGE CHARACTER
    u"\u0007": b"\x07",  # BELL CHARACTER
    u"\u0008": b"\x08",  # BACKSPACE
    u"\u0009": b"\x09",  # HORIZONTAL TAB
    u"\u000A": b"\x0A",  # LINE FEED
    u"\u000B": b"\x0B",  # VERTICAL TAB
    u"\u000C": b"\x0C",  # FORM FEED
    u"\u000D": b"\x0D",  # CARRIAGE RETURN
    u"\u000E": b"\x0E",  # SHIFT OUT
    u"\u000F": b"\x0F",  # SHIFT IN
    u"\u0010": b"\x10",  # DATA LINK ESCAPE
    u"\u0011": b"\x11",  # DEVICE CONTROL 1
    u"\u0012": b"\x12",  # DEVICE CONTROL 2
    u"\u0013": b"\x13",  # DEVICE CONTROL 3
    u"\u0014": b"\x14",  # DEVICE CONTROL 4
    u"\u0015": b"\x15",  # NEGATIVE-ACKNOWLEDGE CHARACTER
    u"\u0016": b"\x16",  # SYNCHRONOUS IDLE
    u"\u0017": b"\x17",  # END OF TRANSMISSION BLOCK
    u"\u0018": b"\x18",  # CANCEL CHARACTER
    u"\u0019": b"\x19",  # END OF MEDIUM
    u"\u001A": b"\x1A",  # SUBSTITUTE CHARACTER
    u"\u001B": b"\x1B",  # ESCAPE CHARACTER
    u"\u001C": b"\x1C",  # FILE SEPARATOR
    u"\u001D": b"\x1D",  # GROUP SEPARATOR
    u"\u001E": b"\x1E",  # RECORD SEPARATOR
    u"\u001F": b"\x1F",  # UNIT SEPARATOR
    u"\u0020": b"\x20",  # SPACE
    u"\u0021": b"\x21",  # EXCLAMATION MARK
    u"\u0022": b"\x22",  # QUOTATION MARK
    u"\u0023": b"\x23",  # NUMBER SIGN
    u"\u0024": b"\x24",  # DOLLAR SIGN
    u"\u0025": b"\x25",  # PERCENT SIGN
    u"\u0026": b"\x26",  # AMPERSAND
    u"\u0027": b"\x27",  # APOSTROPHE
    u"\u0028": b"\x28",  # LEFT PARENTHESIS
    u"\u0029": b"\x29",  # RIGHT PARENTHESIS
    u"\u002A": b"\x2A",  # ASTERISK
    u"\u002B": b"\x2B",  # PLUS SIGN
    u"\u002C": b"\x2C",  # COMMA
    u"\u002D": b"\x2D",  # HYPHEN-MINUS
    u"\u002E": b"\x2E",  # FULL STOP
    u"\u002F": b"\x2F",  # SOLIDUS
    u"\u0030": b"\x30",  # DIGIT ZERO
    u"\u0031": b"\x31",  # DIGIT ONE
    u"\u0032": b"\x32",  # DIGIT TWO
    u"\u0033": b"\x33",  # DIGIT THREE
    u"\u0034": b"\x34",  # DIGIT FOUR
    u"\u0035": b"\x35",  # DIGIT FIVE
    u"\u0036": b"\x36",  # DIGIT SIX
    u"\u0037": b"\x37",  # DIGIT SEVEN
    u"\u0038": b"\x38",  # DIGIT EIGHT
    u"\u0039": b"\x39",  # DIGIT NINE
    u"\u003A": b"\x3A",  # COLON
    u"\u003B": b"\x3B",  # SEMICOLON
    u"\u003C": b"\x3C",  # LESS-THAN SIGN
    u"\u003D": b"\x3D",  # EQUALS SIGN
    u"\u003E": b"\x3E",  # GREATER-THAN SIGN
    u"\u003F": b"\x3F",  # QUESTION MARK
    u"\u0040": b"\x40",  # COMMERCIAL AT
    u"\u0041": b"\x41",  # LATIN CAPITAL LETTER A
    u"\u0042": b"\x42",  # LATIN CAPITAL LETTER B
    u"\u0043": b"\x43",  # LATIN CAPITAL LETTER C
    u"\u0044": b"\x44",  # LATIN CAPITAL LETTER D
    u"\u0045": b"\x45",  # LATIN CAPITAL LETTER E
    u"\u0046": b"\x46",  # LATIN CAPITAL LETTER F
    u"\u0047": b"\x47",  # LATIN CAPITAL LETTER G
    u"\u0048": b"\x48",  # LATIN CAPITAL LETTER H
    u"\u0049": b"\x49",  # LATIN CAPITAL LETTER I
    u"\u004A": b"\x4A",  # LATIN CAPITAL LETTER J
    u"\u004B": b"\x4B",  # LATIN CAPITAL LETTER K
    u"\u004C": b"\x4C",  # LATIN CAPITAL LETTER L
    u"\u004D": b"\x4D",  # LATIN CAPITAL LETTER M
    u"\u004E": b"\x4E",  # LATIN CAPITAL LETTER N
    u"\u004F": b"\x4F",  # LATIN CAPITAL LETTER O
    u"\u0050": b"\x50",  # LATIN CAPITAL LETTER P
    u"\u0051": b"\x51",  # LATIN CAPITAL LETTER Q
    u"\u0052": b"\x52",  # LATIN CAPITAL LETTER R
    u"\u0053": b"\x53",  # LATIN CAPITAL LETTER S
    u"\u0054": b"\x54",  # LATIN CAPITAL LETTER T
    u"\u0055": b"\x55",  # LATIN CAPITAL LETTER U
    u"\u0056": b"\x56",  # LATIN CAPITAL LETTER V
    u"\u0057": b"\x57",  # LATIN CAPITAL LETTER W
    u"\u0058": b"\x58",  # LATIN CAPITAL LETTER X
    u"\u0059": b"\x59",  # LATIN CAPITAL LETTER Y
    u"\u005A": b"\x5A",  # LATIN CAPITAL LETTER Z
    u"\u005B": b"\x5B",  # LEFT SQUARE BRACKET
    u"\u005C": b"\x5C",  # REVERSE SOLIDUS
    u"\u005D": b"\x5D",  # RIGHT SQUARE BRACKET
    u"\u005E": b"\x5E",  # CIRCUMFLEX ACCENT
    u"\u005F": b"\x5F",  # LOW LINE
    u"\u0060": b"\x60",  # GRAVE ACCENT
    u"\u0061": b"\x61",  # LATIN SMALL LETTER A
    u"\u0062": b"\x62",  # LATIN SMALL LETTER B
    u"\u0063": b"\x63",  # LATIN SMALL LETTER C
    u"\u0064": b"\x64",  # LATIN SMALL LETTER D
    u"\u0065": b"\x65",  # LATIN SMALL LETTER E
    u"\u0066": b"\x66",  # LATIN SMALL LETTER F
    u"\u0067": b"\x67",  # LATIN SMALL LETTER G
    u"\u0068": b"\x68",  # LATIN SMALL LETTER H
    u"\u0069": b"\x69",  # LATIN SMALL LETTER I
    u"\u006A": b"\x6A",  # LATIN SMALL LETTER J
    u"\u006B": b"\x6B",  # LATIN SMALL LETTER K
    u"\u006C": b"\x6C",  # LATIN SMALL LETTER L
    u"\u006D": b"\x6D",  # LATIN SMALL LETTER M
    u"\u006E": b"\x6E",  # LATIN SMALL LETTER N
    u"\u006F": b"\x6F",  # LATIN SMALL LETTER O
    u"\u0070": b"\x70",  # LATIN SMALL LETTER P
    u"\u0071": b"\x71",  # LATIN SMALL LETTER Q
    u"\u0072": b"\x72",  # LATIN SMALL LETTER R
    u"\u0073": b"\x73",  # LATIN SMALL LETTER S
    u"\u0074": b"\x74",  # LATIN SMALL LETTER T
    u"\u0075": b"\x75",  # LATIN SMALL LETTER U
    u"\u0076": b"\x76",  # LATIN SMALL LETTER V
    u"\u0077": b"\x77",  # LATIN SMALL LETTER W
    u"\u0078": b"\x78",  # LATIN SMALL LETTER X
    u"\u0079": b"\x79",  # LATIN SMALL LETTER Y
    u"\u007A": b"\x7A",  # LATIN SMALL LETTER Z
    u"\u007B": b"\x7B",  # LEFT CURLY BRACKET
    u"\u007C": b"\x7C",  # VERTICAL LINE
    u"\u007D": b"\x7D",  # RIGHT CURLY BRACKET
    u"\u007E": b"\x7E",  # TILDE
    u"\u007F": b"\x7F",  # DELETE
    u"\u00A1": b"\xC6",  # INVERTED EXCLAMATION MARK
    u"\u00A3": b"\xB9",  # POUND SIGN
    u"\u00A9": b"\xC3",  # COPYRIGHT SIGN
    u"\u00AE": b"\xAA",  # REGISTERED SIGN
    u"\u00B0": b"\xC0",  # DEGREE SIGN
    u"\u00B1": b"\xAB",  # PLUS-MINUS SIGN
    u"\u00B7": b"\xA8",  # MIDDLE DOT
    u"\u00BF": b"\xC5",  # INVERTED QUESTION MARK
    u"\u00C0": b"\xE1\x41",  # LATIN CAPITAL LETTER A WITH GRAVE
    u"\u00C1": b"\xE2\x41",  # LATIN CAPITAL LETTER A WITH ACUTE
    u"\u00C2": b"\xE3\x41",  # LATIN CAPITAL LETTER A WITH CIRCUMFLEX
    u"\u00C3": b"\xE4\x41",  # LATIN CAPITAL LETTER A WITH TILDE
    u"\u00C4": b"\xE8\x41",  # LATIN CAPITAL LETTER A WITH DIAERESIS
    u"\u00C5": b"\xEA\x41",  # LATIN CAPITAL LETTER A WITH RING ABOVE
    u"\u00C6": b"\xA5",  # LATIN CAPITAL LETTER AE
    u"\u00C7": b"\xF0\x43",  # LATIN CAPITAL LETTER C WITH CEDILLA
    u"\u00C8": b"\xE1\x45",  # LATIN CAPITAL LETTER E WITH GRAVE
    u"\u00C9": b"\xE2\x45",  # LATIN CAPITAL LETTER E WITH ACUTE
    u"\u00CA": b"\xE3\x45",  # LATIN CAPITAL LETTER E WITH CIRCUMFLEX
    u"\u00CB": b"\xE8\x45",  # LATIN CAPITAL LETTER E WITH DIAERESIS
    u"\u00CC": b"\xE1\x49",  # LATIN CAPITAL LETTER I WITH GRAVE
    u"\u00CD": b"\xE2\x49",  # LATIN CAPITAL LETTER I WITH ACUTE
    u"\u00CE": b"\xE3\x49",  # LATIN CAPITAL LETTER I WITH CIRCUMFLEX
    u"\u00CF": b"\xE8\x49",  # LATIN CAPITAL LETTER I WITH DIAERESIS
    u"\u00D1": b"\xE4\x4E",  # LATIN CAPITAL LETTER N WITH TILDE
    u"\u00D2": b"\xE1\x4F",  # LATIN CAPITAL LETTER O WITH GRAVE
    u"\u00D3": b"\xE2\x4F",  # LATIN CAPITAL LETTER O WITH ACUTE
    u"\u00D4": b"\xE3\x4F",  # LATIN CAPITAL LETTER O WITH CIRCUMFLEX
    u"\u00D5": b"\xE4\x4F",  # LATIN CAPITAL LETTER O WITH TILDE
    u"\u00D6": b"\xE8\x4F",  # LATIN CAPITAL LETTER O WITH DIAERESIS
    u"\u00D8": b"\xA2",  # LATIN CAPITAL LETTER O WITH STROKE
    u"\u00D9": b"\xE1\x55",  # LATIN CAPITAL LETTER U WITH GRAVE
    u"\u00DA": b"\xE2\x55",  # LATIN CAPITAL LETTER U WITH ACUTE
    u"\u00DB": b"\xE3\x55",  # LATIN CAPITAL LETTER U WITH CIRCUMFLEX
    u"\u00DC": b"\xE8\x55",  # LATIN CAPITAL LETTER U WITH DIAERESIS
    u"\u00DD": b"\xE2\x59",  # LATIN CAPITAL LETTER Y WITH ACUTE
    u"\u00DE": b"\xA4",  # LATIN CAPITAL LETTER THORN
    u"\u00E0": b"\xE1\x61",  # LATIN SMALL LETTER A WITH GRAVE
    u"\u00E1": b"\xE2\x61",  # LATIN SMALL LETTER A WITH ACUTE
    u"\u00E2": b"\xE3\x61",  # LATIN SMALL LETTER A WITH CIRCUMFLEX
    u"\u00E3": b"\xE4\x61",  # LATIN SMALL LETTER A WITH TILDE
    u"\u00E4": b"\xE8\x61",  # LATIN SMALL LETTER A WITH DIAERESIS
    u"\u00E5": b"\xEA\x61",  # LATIN SMALL LETTER A WITH RING ABOVE
    u"\u00E6": b"\xB5",  # LATIN SMALL LETTER AE
    u"\u00E7": b"\xF0\x63",  # LATIN SMALL LETTER C WITH CEDILLA
    u"\u00E8": b"\xE1\x65",  # LATIN SMALL LETTER E WITH GRAVE
    u"\u00E9": b"\xE2\x65",  # LATIN SMALL LETTER E WITH ACUTE
    u"\u00EA": b"\xE3\x65",  # LATIN SMALL LETTER E WITH CIRCUMFLEX
    u"\u00EB": b"\xE8\x65",  # LATIN SMALL LETTER E WITH DIAERESIS
    u"\u00EC": b"\xE1\x69",  # LATIN SMALL LETTER I WITH GRAVE
    u"\u00ED": b"\xE2\x69",  # LATIN SMALL LETTER I WITH ACUTE
    u"\u00EE": b"\xE3\x69",  # LATIN SMALL LETTER I WITH CIRCUMFLEX
    u"\u00EF": b"\xE8\x69",  # LATIN SMALL LETTER I WITH DIAERESIS
    u"\u00F0": b"\xBA",  # LATIN SMALL LETTER ETH
    u"\u00F1": b"\xE4\x6E",  # LATIN SMALL LETTER N WITH TILDE
    u"\u00F2": b"\xE1\x6F",  # LATIN SMALL LETTER O WITH GRAVE
    u"\u00F3": b"\xE2\x6F",  # LATIN SMALL LETTER O WITH ACUTE
    u"\u00F4": b"\xE3\x6F",  # LATIN SMALL LETTER O WITH CIRCUMFLEX
    u"\u00F5": b"\xE4\x6F",  # LATIN SMALL LETTER O WITH TILDE
    u"\u00F6": b"\xE8\x6F",  # LATIN SMALL LETTER O WITH DIAERESIS
    u"\u00F8": b"\xB2",  # LATIN SMALL LETTER O WITH STROKE
    u"\u00F9": b"\xE1\x75",  # LATIN SMALL LETTER U WITH GRAVE
    u"\u00FA": b"\xE2\x75",  # LATIN SMALL LETTER U WITH ACUTE
    u"\u00FB": b"\xE3\x75",  # LATIN SMALL LETTER U WITH CIRCUMFLEX
    u"\u00FC": b"\xE8\x75",  # LATIN SMALL LETTER U WITH DIAERESIS
    u"\u00FD": b"\xE2\x79",  # LATIN SMALL LETTER Y WITH ACUTE
    u"\u00FE": b"\xB4",  # LATIN SMALL LETTER THORN
    u"\u00FF": b"\xE8\x79",  # LATIN SMALL LETTER Y WITH DIAERESIS
    u"\u0100": b"\xE5\x41",  # LATIN CAPITAL LETTER A WITH MACRON
    u"\u0101": b"\xE5\x61",  # LATIN SMALL LETTER A WITH MACRON
    u"\u0102": b"\xE6\x41",  # LATIN CAPITAL LETTER A WITH BREVE
    u"\u0103": b"\xE6\x61",  # LATIN SMALL LETTER A WITH BREVE
    u"\u0104": b"\xF1\x41",  # LATIN CAPITAL LETTER A WITH OGONEK
    u"\u0105": b"\xF1\x61",  # LATIN SMALL LETTER A WITH OGONEK
    u"\u0106": b"\xE2\x43",  # LATIN CAPITAL LETTER C WITH ACUTE
    u"\u0107": b"\xE2\x63",  # LATIN SMALL LETTER C WITH ACUTE
    u"\u0108": b"\xE3\x43",  # LATIN CAPITAL LETTER C WITH CIRCUMFLEX
    u"\u0109": b"\xE3\x63",  # LATIN SMALL LETTER C WITH CIRCUMFLEX
    u"\u010A": b"\xE7\x43",  # LATIN CAPITAL LETTER C WITH DOT ABOVE
    u"\u010B": b"\xE7\x63",  # LATIN SMALL LETTER C WITH DOT ABOVE
    u"\u010C": b"\xE9\x43",  # LATIN CAPITAL LETTER C WITH CARON
    u"\u010D": b"\xE9\x63",  # LATIN SMALL LETTER C WITH CARON
    u"\u010E": b"\xE9\x44",  # LATIN CAPITAL LETTER D WITH CARON
    u"\u010F": b"\xE9\x64",  # LATIN SMALL LETTER D WITH CARON
    u"\u0110": b"\xA3",  # LATIN CAPITAL LETTER D WITH STROKE
    u"\u0111": b"\xB3",  # LATIN SMALL LETTER D WITH STROKE
    u"\u0112": b"\xE5\x45",  # LATIN CAPITAL LETTER E WITH MACRON
    u"\u0113": b"\xE5\x65",  # LATIN SMALL LETTER E WITH MACRON
    u"\u0114": b"\xE6\x45",  # LATIN CAPITAL LETTER E WITH BREVE
    u"\u0115": b"\xE6\x65",  # LATIN SMALL LETTER E WITH BREVE
    u"\u0116": b"\xE7\x45",  # LATIN CAPITAL LETTER E WITH DOT ABOVE
    u"\u0117": b"\xE7\x65",  # LATIN SMALL LETTER E WITH DOT ABOVE
    u"\u0118": b"\xF1\x45",  # LATIN CAPITAL LETTER E WITH OGONEK
    u"\u0119": b"\xF1\x65",  # LATIN SMALL LETTER E WITH OGONEK
    u"\u011A": b"\xE9\x45",  # LATIN CAPITAL LETTER E WITH CARON
    u"\u011B": b"\xE9\x65",  # LATIN SMALL LETTER E WITH CARON
    u"\u011C": b"\xE3\x47",  # LATIN CAPITAL LETTER G WITH CIRCUMFLEX
    u"\u011D": b"\xE3\x67",  # LATIN SMALL LETTER G WITH CIRCUMFLEX
    u"\u011E": b"\xE6\x47",  # LATIN CAPITAL LETTER G WITH BREVE
    u"\u011F": b"\xE6\x67",  # LATIN SMALL LETTER G WITH BREVE
    u"\u0120": b"\xE7\x47",  # LATIN CAPITAL LETTER G WITH DOT ABOVE
    u"\u0121": b"\xE7\x67",  # LATIN SMALL LETTER G WITH DOT ABOVE
    u"\u0122": b"\xF0\x47",  # LATIN CAPITAL LETTER G WITH CEDILLA
    u"\u0123": b"\xF0\x67",  # LATIN SMALL LETTER G WITH CEDILLA
    u"\u0124": b"\xE3\x48",  # LATIN CAPITAL LETTER H WITH CIRCUMFLEX
    u"\u0125": b"\xE3\x68",  # LATIN SMALL LETTER H WITH CIRCUMFLEX
    u"\u0128": b"\xE4\x49",  # LATIN CAPITAL LETTER I WITH TILDE
    u"\u0129": b"\xE4\x69",  # LATIN SMALL LETTER I WITH TILDE
    u"\u012A": b"\xE5\x49",  # LATIN CAPITAL LETTER I WITH MACRON
    u"\u012B": b"\xE5\x69",  # LATIN SMALL LETTER I WITH MACRON
    u"\u012C": b"\xE6\x49",  # LATIN CAPITAL LETTER I WITH BREVE
    u"\u012D": b"\xE6\x69",  # LATIN SMALL LETTER I WITH BREVE
    u"\u012E": b"\xF1\x49",  # LATIN CAPITAL LETTER I WITH OGONEK
    u"\u012F": b"\xF1\x69",  # LATIN SMALL LETTER I WITH OGONEK
    u"\u0130": b"\xE7\x49",  # LATIN CAPITAL LETTER I WITH DOT ABOVE
    u"\u0131": b"\xB8",  # LATIN SMALL LETTER DOTLESS I
    u"\u0134": b"\xE3\x4A",  # LATIN CAPITAL LETTER J WITH CIRCUMFLEX
    u"\u0135": b"\xE3\x6A",  # LATIN SMALL LETTER J WITH CIRCUMFLEX
    u"\u0136": b"\xF0\x4B",  # LATIN CAPITAL LETTER K WITH CEDILLA
    u"\u0137": b"\xF0\x6B",  # LATIN SMALL LETTER K WITH CEDILLA
    u"\u0139": b"\xE2\x4C",  # LATIN CAPITAL LETTER L WITH ACUTE
    u"\u013A": b"\xE2\x6C",  # LATIN SMALL LETTER L WITH ACUTE
    u"\u013B": b"\xF0\x4C",  # LATIN CAPITAL LETTER L WITH CEDILLA
    u"\u013C": b"\xF0\x6C",  # LATIN SMALL LETTER L WITH CEDILLA
    u"\u013D": b"\xE9\x4C",  # LATIN CAPITAL LETTER L WITH CARON
    u"\u013E": b"\xE9\x6C",  # LATIN SMALL LETTER L WITH CARON
    u"\u0141": b"\xA1",  # LATIN CAPITAL LETTER L WITH STROKE
    u"\u0142": b"\xB1",  # LATIN SMALL LETTER L WITH STROKE
    u"\u0143": b"\xE2\x4E",  # LATIN CAPITAL LETTER N WITH ACUTE
    u"\u0144": b"\xE2\x6E",  # LATIN SMALL LETTER N WITH ACUTE
    u"\u0145": b"\xF0\x4E",  # LATIN CAPITAL LETTER N WITH CEDILLA
    u"\u0146": b"\xF0\x6E",  # LATIN SMALL LETTER N WITH CEDILLA
    u"\u0147": b"\xE9\x4E",  # LATIN CAPITAL LETTER N WITH CARON
    u"\u0148": b"\xE9\x6E",  # LATIN SMALL LETTER N WITH CARON
    u"\u014C": b"\xE5\x4F",  # LATIN CAPITAL LETTER O WITH MACRON
    u"\u014D": b"\xE5\x6F",  # LATIN SMALL LETTER O WITH MACRON
    u"\u014E": b"\xE6\x4F",  # LATIN CAPITAL LETTER O WITH BREVE
    u"\u014F": b"\xE6\x6F",  # LATIN SMALL LETTER O WITH BREVE
    u"\u0150": b"\xEE\x4F",  # LATIN CAPITAL LETTER O WITH DOUBLE ACUTE
    u"\u0151": b"\xEE\x6F",  # LATIN SMALL LETTER O WITH DOUBLE ACUTE
    u"\u0152": b"\xA6",  # LATIN CAPITAL LIGATURE OE
    u"\u0153": b"\xB6",  # LATIN SMALL LIGATURE OE
    u"\u0154": b"\xE2\x52",  # LATIN CAPITAL LETTER R WITH ACUTE
    u"\u0155": b"\xE2\x72",  # LATIN SMALL LETTER R WITH ACUTE
    u"\u0156": b"\xF0\x52",  # LATIN CAPITAL LETTER R WITH CEDILLA
    u"\u0157": b"\xF0\x72",  # LATIN SMALL LETTER R WITH CEDILLA
    u"\u0158": b"\xE9\x52",  # LATIN CAPITAL LETTER R WITH CARON
    u"\u0159": b"\xE9\x72",  # LATIN SMALL LETTER R WITH CARON
    u"\u015A": b"\xE2\x53",  # LATIN CAPITAL LETTER S WITH ACUTE
    u"\u015B": b"\xE2\x73",  # LATIN SMALL LETTER S WITH ACUTE
    u"\u015C": b"\xE3\x53",  # LATIN CAPITAL LETTER S WITH CIRCUMFLEX
    u"\u015D": b"\xE3\x73",  # LATIN SMALL LETTER S WITH CIRCUMFLEX
    u"\u015E": b"\xF0\x53",  # LATIN CAPITAL LETTER S WITH CEDILLA
    u"\u015F": b"\xF0\x73",  # LATIN SMALL LETTER S WITH CEDILLA
    u"\u0160": b"\xE9\x53",  # LATIN CAPITAL LETTER S WITH CARON
    u"\u0161": b"\xE9\x73",  # LATIN SMALL LETTER S WITH CARON
    u"\u0162": b"\xF0\x54",  # LATIN CAPITAL LETTER T WITH CEDILLA
    u"\u0163": b"\xF0\x74",  # LATIN SMALL LETTER T WITH CEDILLA
    u"\u0164": b"\xE9\x54",  # LATIN CAPITAL LETTER T WITH CARON
    u"\u0165": b"\xE9\x74",  # LATIN SMALL LETTER T WITH CARON
    u"\u0168": b"\xE4\x55",  # LATIN CAPITAL LETTER U WITH TILDE
    u"\u0169": b"\xE4\x75",  # LATIN SMALL LETTER U WITH TILDE
    u"\u016A": b"\xE5\x55",  # LATIN CAPITAL LETTER U WITH MACRON
    u"\u016B": b"\xE5\x75",  # LATIN SMALL LETTER U WITH MACRON
    u"\u016C": b"\xE6\x55",  # LATIN CAPITAL LETTER U WITH BREVE
    u"\u016D": b"\xE6\x75",  # LATIN SMALL LETTER U WITH BREVE
    u"\u016E": b"\xEA\x55",  # LATIN CAPITAL LETTER U WITH RING ABOVE
    u"\u016F": b"\xEA\x75",  # LATIN SMALL LETTER U WITH RING ABOVE
    u"\u0170": b"\xEE\x55",  # LATIN CAPITAL LETTER U WITH DOUBLE ACUTE
    u"\u0171": b"\xEE\x75",  # LATIN SMALL LETTER U WITH DOUBLE ACUTE
    u"\u0172": b"\xF1\x55",  # LATIN CAPITAL LETTER U WITH OGONEK
    u"\u0173": b"\xF1\x75",  # LATIN SMALL LETTER U WITH OGONEK
    u"\u0174": b"\xE3\x57",  # LATIN CAPITAL LETTER W WITH CIRCUMFLEX
    u"\u0175": b"\xE3\x77",  # LATIN SMALL LETTER W WITH CIRCUMFLEX
    u"\u0176": b"\xE3\x59",  # LATIN CAPITAL LETTER Y WITH CIRCUMFLEX
    u"\u0177": b"\xE3\x79",  # LATIN SMALL LETTER Y WITH CIRCUMFLEX
    u"\u0178": b"\xE8\x59",  # LATIN CAPITAL LETTER Y WITH DIAERESIS
    u"\u0179": b"\xE2\x5A",  # LATIN CAPITAL LETTER Z WITH ACUTE
    u"\u017A": b"\xE2\x7A",  # LATIN SMALL LETTER Z WITH ACUTE
    u"\u017B": b"\xE7\x5A",  # LATIN CAPITAL LETTER Z WITH DOT ABOVE
    u"\u017C": b"\xE7\x7A",  # LATIN SMALL LETTER Z WITH DOT ABOVE
    u"\u017D": b"\xE9\x5A",  # LATIN CAPITAL LETTER Z WITH CARON
    u"\u017E": b"\xE9\x7A",  # LATIN SMALL LETTER Z WITH CARON
    u"\u01A0": b"\xAC",  # LATIN CAPITAL LETTER O WITH HORN
    u"\u01A1": b"\xBC",  # LATIN SMALL LETTER O WITH HORN
    u"\u01AF": b"\xAD",  # LATIN CAPITAL LETTER U WITH HORN
    u"\u01B0": b"\xBD",  # LATIN SMALL LETTER U WITH HORN
    u"\u01CD": b"\xE9\x41",  # LATIN CAPITAL LETTER A WITH CARON
    u"\u01CE": b"\xE9\x61",  # LATIN SMALL LETTER A WITH CARON
    u"\u01CF": b"\xE9\x49",  # LATIN CAPITAL LETTER I WITH CARON
    u"\u01D0": b"\xE9\x69",  # LATIN SMALL LETTER I WITH CARON
    u"\u01D1": b"\xE9\x4F",  # LATIN CAPITAL LETTER O WITH CARON
    u"\u01D2": b"\xE9\x6F",  # LATIN SMALL LETTER O WITH CARON
    u"\u01D3": b"\xE9\x55",  # LATIN CAPITAL LETTER U WITH CARON
    u"\u01D4": b"\xE9\x75",  # LATIN SMALL LETTER U WITH CARON
    u"\u01D5": b"\xE5\xE8\x55",  # LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON
    u"\u01D6": b"\xE5\xE8\x75",  # LATIN SMALL LETTER U WITH DIAERESIS AND MACRON
    u"\u01D7": b"\xE2\xE8\x55",  # LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE
    u"\u01D8": b"\xE2\xE8\x75",  # LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE
    u"\u01D9": b"\xE9\xE8\x55",  # LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON
    u"\u01DA": b"\xE9\xE8\x75",  # LATIN SMALL LETTER U WITH DIAERESIS AND CARON
    u"\u01DB": b"\xE1\xE8\x55",  # LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE
    u"\u01DC": b"\xE1\xE8\x75",  # LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE
    u"\u01DE": b"\xE5\xE8\x41",  # LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON
    u"\u01DF": b"\xE5\xE8\x61",  # LATIN SMALL LETTER A WITH DIAERESIS AND MACRON
    u"\u01E0": b"\xE5\xE7\x41",  # LATIN CAPITAL LETTER A WITH DOT ABOVE AND MACRON
    u"\u01E1": b"\xE5\xE7\x61",  # LATIN SMALL LETTER A WITH DOT ABOVE AND MACRON
    u"\u01E2": b"\xE5\xA5",  # LATIN CAPITAL LETTER AE WITH MACRON
    u"\u01E3": b"\xE5\xB5",  # LATIN SMALL LETTER AE WITH MACRON
    u"\u01E6": b"\xE9\x47",  # LATIN CAPITAL LETTER G WITH CARON
    u"\u01E7": b"\xE9\x67",  # LATIN SMALL LETTER G WITH CARON
    u"\u01E8": b"\xE9\x4B",  # LATIN CAPITAL LETTER K WITH CARON
    u"\u01E9": b"\xE9\x6B",  # LATIN SMALL LETTER K WITH CARON
    u"\u01EA": b"\xF1\x4F",  # LATIN CAPITAL LETTER O WITH OGONEK
    u"\u01EB": b"\xF1\x6F",  # LATIN SMALL LETTER O WITH OGONEK
    u"\u01EC": b"\xE5\xF1\x4F",  # LATIN CAPITAL LETTER O WITH OGONEK AND MACRON
    u"\u01ED": b"\xE5\xF1\x6F",  # LATIN SMALL LETTER O WITH OGONEK AND MACRON
    u"\u01F0": b"\xE9\x6A",  # LATIN SMALL LETTER J WITH CARON
    u"\u01F4": b"\xE2\x47",  # LATIN CAPITAL LETTER G WITH ACUTE
    u"\u01F5": b"\xE2\x67",  # LATIN SMALL LETTER G WITH ACUTE
    u"\u01F8": b"\xE1\x4E",  # LATIN CAPITAL LETTER N WITH GRAVE
    u"\u01F9": b"\xE1\x6E",  # LATIN SMALL LETTER N WITH GRAVE
    u"\u01FA": b"\xE2\xEA\x41",  # LATIN CAPITAL LETTER A WITH RING ABOVE AND ACUTE
    u"\u01FB": b"\xE2\xEA\x61",  # LATIN SMALL LETTER A WITH RING ABOVE AND ACUTE
    u"\u01FC": b"\xE2\xA5",  # LATIN CAPITAL LETTER AE WITH ACUTE
    u"\u01FD": b"\xE2\xB5",  # LATIN SMALL LETTER AE WITH ACUTE
    u"\u01FE": b"\xE2\xA2",  # LATIN CAPITAL LETTER O WITH STROKE AND ACUTE
    u"\u01FF": b"\xE2\xB2",  # LATIN SMALL LETTER O WITH STROKE AND ACUTE
    u"\u0218": b"\xF7\x53",  # LATIN CAPITAL LETTER S WITH COMMA BELOW
    u"\u0219": b"\xF7\x73",  # LATIN SMALL LETTER S WITH COMMA BELOW
    u"\u021A": b"\xF7\x54",  # LATIN CAPITAL LETTER T WITH COMMA BELOW
    u"\u021B": b"\xF7\x74",  # LATIN SMALL LETTER T WITH COMMA BELOW
    u"\u021E": b"\xE9\x48",  # LATIN CAPITAL LETTER H WITH CARON
    u"\u021F": b"\xE9\x68",  # LATIN SMALL LETTER H WITH CARON
    u"\u0226": b"\xE7\x41",  # LATIN CAPITAL LETTER A WITH DOT ABOVE
    u"\u0227": b"\xE7\x61",  # LATIN SMALL LETTER A WITH DOT ABOVE
    u"\u0228": b"\xF0\x45",  # LATIN CAPITAL LETTER E WITH CEDILLA
    u"\u0229": b"\xF0\x65",  # LATIN SMALL LETTER E WITH CEDILLA
    u"\u022A": b"\xE5\xE8\x4F",  # LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON
    u"\u022B": b"\xE5\xE8\x6F",  # LATIN SMALL LETTER O WITH DIAERESIS AND MACRON
    u"\u022C": b"\xE5\xE4\x4F",  # LATIN CAPITAL LETTER O WITH TILDE AND MACRON
    u"\u022D": b"\xE5\xE4\x6F",  # LATIN SMALL LETTER O WITH TILDE AND MACRON
    u"\u022E": b"\xE7\x4F",  # LATIN CAPITAL LETTER O WITH DOT ABOVE
    u"\u022F": b"\xE7\x6F",  # LATIN SMALL LETTER O WITH DOT ABOVE
    u"\u0230": b"\xE5\xE7\x4F",  # LATIN CAPITAL LETTER O WITH DOT ABOVE AND MACRON
    u"\u0231": b"\xE5\xE7\x6F",  # LATIN SMALL LETTER O WITH DOT ABOVE AND MACRON
    u"\u0232": b"\xE5\x59",  # LATIN CAPITAL LETTER Y WITH MACRON
    u"\u0233": b"\xE5\x79",  # LATIN SMALL LETTER Y WITH MACRON
    u"\u02B9": b"\xA7",  # MODIFIER LETTER PRIME
    u"\u02BA": b"\xB7",  # MODIFIER LETTER DOUBLE PRIME
    u"\u02BB": b"\xB0",  # MODIFIER LETTER TURNED COMMA
    u"\u02BC": b"\xAE",  # MODIFIER LETTER APOSTROPHE
    u"\u0374": b"\xA7",  # GREEK NUMERAL SIGN
    u"\u037E": b"\x3B",  # GREEK QUESTION MARK
    u"\u0387": b"\xA8",  # GREEK ANO TELEIA
    u"\u1E00": b"\xF4\x41",  # LATIN CAPITAL LETTER A WITH RING BELOW
    u"\u1E01": b"\xF4\x61",  # LATIN SMALL LETTER A WITH RING BELOW
    u"\u1E02": b"\xE7\x42",  # LATIN CAPITAL LETTER B WITH DOT ABOVE
    u"\u1E03": b"\xE7\x62",  # LATIN SMALL LETTER B WITH DOT ABOVE
    u"\u1E04": b"\xF2\x42",  # LATIN CAPITAL LETTER B WITH DOT BELOW
    u"\u1E05": b"\xF2\x62",  # LATIN SMALL LETTER B WITH DOT BELOW
    u"\u1E08": b"\xE2\xF0\x43",  # LATIN CAPITAL LETTER C WITH CEDILLA AND ACUTE
    u"\u1E09": b"\xE2\xF0\x63",  # LATIN SMALL LETTER C WITH CEDILLA AND ACUTE
    u"\u1E0A": b"\xE7\x44",  # LATIN CAPITAL LETTER D WITH DOT ABOVE
    u"\u1E0B": b"\xE7\x64",  # LATIN SMALL LETTER D WITH DOT ABOVE
    u"\u1E0C": b"\xF2\x44",  # LATIN CAPITAL LETTER D WITH DOT BELOW
    u"\u1E0D": b"\xF2\x64",  # LATIN SMALL LETTER D WITH DOT BELOW
    u"\u1E10": b"\xF0\x44",  # LATIN CAPITAL LETTER D WITH CEDILLA
    u"\u1E11": b"\xF0\x64",  # LATIN SMALL LETTER D WITH CEDILLA
    u"\u1E14": b"\xE1\xE5\x45",  # LATIN CAPITAL LETTER E WITH MACRON AND GRAVE
    u"\u1E15": b"\xE1\xE5\x65",  # LATIN SMALL LETTER E WITH MACRON AND GRAVE
    u"\u1E16": b"\xE2\xE5\x45",  # LATIN CAPITAL LETTER E WITH MACRON AND ACUTE
    u"\u1E17": b"\xE2\xE5\x65",  # LATIN SMALL LETTER E WITH MACRON AND ACUTE
    u"\u1E1C": b"\xE6\xF0\x45",  # LATIN CAPITAL LETTER E WITH CEDILLA AND BREVE
    u"\u1E1D": b"\xE6\xF0\x65",  # LATIN SMALL LETTER E WITH CEDILLA AND BREVE
    u"\u1E1E": b"\xE7\x46",  # LATIN CAPITAL LETTER F WITH DOT ABOVE
    u"\u1E1F": b"\xE7\x66",  # LATIN SMALL LETTER F WITH DOT ABOVE
    u"\u1E20": b"\xE5\x47",  # LATIN CAPITAL LETTER G WITH MACRON
    u"\u1E21": b"\xE5\x67",  # LATIN SMALL LETTER G WITH MACRON
    u"\u1E22": b"\xE7\x48",  # LATIN CAPITAL LETTER H WITH DOT ABOVE
    u"\u1E23": b"\xE7\x68",  # LATIN SMALL LETTER H WITH DOT ABOVE
    u"\u1E24": b"\xF2\x48",  # LATIN CAPITAL LETTER H WITH DOT BELOW
    u"\u1E25": b"\xF2\x68",  # LATIN SMALL LETTER H WITH DOT BELOW
    u"\u1E26": b"\xE8\x48",  # LATIN CAPITAL LETTER H WITH DIAERESIS
    u"\u1E27": b"\xE8\x68",  # LATIN SMALL LETTER H WITH DIAERESIS
    u"\u1E28": b"\xF0\x48",  # LATIN CAPITAL LETTER H WITH CEDILLA
    u"\u1E29": b"\xF0\x68",  # LATIN SMALL LETTER H WITH CEDILLA
    u"\u1E2A": b"\xF9\x48",  # LATIN CAPITAL LETTER H WITH BREVE BELOW
    u"\u1E2B": b"\xF9\x68",  # LATIN SMALL LETTER H WITH BREVE BELOW
    u"\u1E2E": b"\xE2\xE8\x49",  # LATIN CAPITAL LETTER I WITH DIAERESIS AND ACUTE
    u"\u1E2F": b"\xE2\xE8\x69",  # LATIN SMALL LETTER I WITH DIAERESIS AND ACUTE
    u"\u1E30": b"\xE2\x4B",  # LATIN CAPITAL LETTER K WITH ACUTE
    u"\u1E31": b"\xE2\x6B",  # LATIN SMALL LETTER K WITH ACUTE
    u"\u1E32": b"\xF2\x4B",  # LATIN CAPITAL LETTER K WITH DOT BELOW
    u"\u1E33": b"\xF2\x6B",  # LATIN SMALL LETTER K WITH DOT BELOW
    u"\u1E36": b"\xF2\x4C",  # LATIN CAPITAL LETTER L WITH DOT BELOW
    u"\u1E37": b"\xF2\x6C",  # LATIN SMALL LETTER L WITH DOT BELOW
    u"\u1E38": b"\xE5\xF2\x4C",  # LATIN CAPITAL LETTER L WITH DOT BELOW AND MACRON
    u"\u1E39": b"\xE5\xF2\x6C",  # LATIN SMALL LETTER L WITH DOT BELOW AND MACRON
    u"\u1E3E": b"\xE2\x4D",  # LATIN CAPITAL LETTER M WITH ACUTE
    u"\u1E3F": b"\xE2\x6D",  # LATIN SMALL LETTER M WITH ACUTE
    u"\u1E40": b"\xE7\x4D",  # LATIN CAPITAL LETTER M WITH DOT ABOVE
    u"\u1E41": b"\xE7\x6D",  # LATIN SMALL LETTER M WITH DOT ABOVE
    u"\u1E42": b"\xF2\x4D",  # LATIN CAPITAL LETTER M WITH DOT BELOW
    u"\u1E43": b"\xF2\x6D",  # LATIN SMALL LETTER M WITH DOT BELOW
    u"\u1E44": b"\xE7\x4E",  # LATIN CAPITAL LETTER N WITH DOT ABOVE
    u"\u1E45": b"\xE7\x6E",  # LATIN SMALL LETTER N WITH DOT ABOVE
    u"\u1E46": b"\xF2\x4E",  # LATIN CAPITAL LETTER N WITH DOT BELOW
    u"\u1E47": b"\xF2\x6E",  # LATIN SMALL LETTER N WITH DOT BELOW
    u"\u1E4C": b"\xE2\xE4\x4F",  # LATIN CAPITAL LETTER O WITH TILDE AND ACUTE
    u"\u1E4D": b"\xE2\xE4\x6F",  # LATIN SMALL LETTER O WITH TILDE AND ACUTE
    u"\u1E4E": b"\xE8\xE4\x4F",  # LATIN CAPITAL LETTER O WITH TILDE AND DIAERESIS
    u"\u1E4F": b"\xE8\xE4\x6F",  # LATIN SMALL LETTER O WITH TILDE AND DIAERESIS
    u"\u1E50": b"\xE1\xE5\x4F",  # LATIN CAPITAL LETTER O WITH MACRON AND GRAVE
    u"\u1E51": b"\xE1\xE5\x6F",  # LATIN SMALL LETTER O WITH MACRON AND GRAVE
    u"\u1E52": b"\xE2\xE5\x4F",  # LATIN CAPITAL LETTER O WITH MACRON AND ACUTE
    u"\u1E53": b"\xE2\xE5\x6F",  # LATIN SMALL LETTER O WITH MACRON AND ACUTE
    u"\u1E54": b"\xE2\x50",  # LATIN CAPITAL LETTER P WITH ACUTE
    u"\u1E55": b"\xE2\x70",  # LATIN SMALL LETTER P WITH ACUTE
    u"\u1E56": b"\xE7\x50",  # LATIN CAPITAL LETTER P WITH DOT ABOVE
    u"\u1E57": b"\xE7\x70",  # LATIN SMALL LETTER P WITH DOT ABOVE
    u"\u1E58": b"\xE7\x52",  # LATIN CAPITAL LETTER R WITH DOT ABOVE
    u"\u1E59": b"\xE7\x72",  # LATIN SMALL LETTER R WITH DOT ABOVE
    u"\u1E5A": b"\xF2\x52",  # LATIN CAPITAL LETTER R WITH DOT BELOW
    u"\u1E5B": b"\xF2\x72",  # LATIN SMALL LETTER R WITH DOT BELOW
    u"\u1E5C": b"\xE5\xF2\x52",  # LATIN CAPITAL LETTER R WITH DOT BELOW AND MACRON
    u"\u1E5D": b"\xE5\xF2\x72",  # LATIN SMALL LETTER R WITH DOT BELOW AND MACRON
    u"\u1E60": b"\xE7\x53",  # LATIN CAPITAL LETTER S WITH DOT ABOVE
    u"\u1E61": b"\xE7\x73",  # LATIN SMALL LETTER S WITH DOT ABOVE
    u"\u1E62": b"\xF2\x53",  # LATIN CAPITAL LETTER S WITH DOT BELOW
    u"\u1E63": b"\xF2\x73",  # LATIN SMALL LETTER S WITH DOT BELOW
    u"\u1E64": b"\xE7\xE2\x53",  # LATIN CAPITAL LETTER S WITH ACUTE AND DOT ABOVE
    u"\u1E65": b"\xE7\xE2\x73",  # LATIN SMALL LETTER S WITH ACUTE AND DOT ABOVE
    u"\u1E66": b"\xE7\xE9\x53",  # LATIN CAPITAL LETTER S WITH CARON AND DOT ABOVE
    u"\u1E67": b"\xE7\xE9\x73",  # LATIN SMALL LETTER S WITH CARON AND DOT ABOVE
    u"\u1E68": b"\xE7\xF2\x53",  # LATIN CAPITAL LETTER S WITH DOT BELOW AND DOT ABOVE
    u"\u1E69": b"\xE7\xF2\x73",  # LATIN SMALL LETTER S WITH DOT BELOW AND DOT ABOVE
    u"\u1E6A": b"\xE7\x54",  # LATIN CAPITAL LETTER T WITH DOT ABOVE
    u"\u1E6B": b"\xE7\x74",  # LATIN SMALL LETTER T WITH DOT ABOVE
    u"\u1E6C": b"\xF2\x54",  # LATIN CAPITAL LETTER T WITH DOT BELOW
    u"\u1E6D": b"\xF2\x74",  # LATIN SMALL LETTER T WITH DOT BELOW
    u"\u1E72": b"\xF3\x55",  # LATIN CAPITAL LETTER U WITH DIAERESIS BELOW
    u"\u1E73": b"\xF3\x75",  # LATIN SMALL LETTER U WITH DIAERESIS BELOW
    u"\u1E78": b"\xE2\xE4\x55",  # LATIN CAPITAL LETTER U WITH TILDE AND ACUTE
    u"\u1E79": b"\xE2\xE4\x75",  # LATIN SMALL LETTER U WITH TILDE AND ACUTE
    u"\u1E7A": b"\xE8\xE5\x55",  # LATIN CAPITAL LETTER U WITH MACRON AND DIAERESIS
    u"\u1E7B": b"\xE8\xE5\x75",  # LATIN SMALL LETTER U WITH MACRON AND DIAERESIS
    u"\u1E7C": b"\xE4\x56",  # LATIN CAPITAL LETTER V WITH TILDE
    u"\u1E7D": b"\xE4\x76",  # LATIN SMALL LETTER V WITH TILDE
    u"\u1E7E": b"\xF2\x56",  # LATIN CAPITAL LETTER V WITH DOT BELOW
    u"\u1E7F": b"\xF2\x76",  # LATIN SMALL LETTER V WITH DOT BELOW
    u"\u1E80": b"\xE1\x57",  # LATIN CAPITAL LETTER W WITH GRAVE
    u"\u1E81": b"\xE1\x77",  # LATIN SMALL LETTER W WITH GRAVE
    u"\u1E82": b"\xE2\x57",  # LATIN CAPITAL LETTER W WITH ACUTE
    u"\u1E83": b"\xE2\x77",  # LATIN SMALL LETTER W WITH ACUTE
    u"\u1E84": b"\xE8\x57",  # LATIN CAPITAL LETTER W WITH DIAERESIS
    u"\u1E85": b"\xE8\x77",  # LATIN SMALL LETTER W WITH DIAERESIS
    u"\u1E86": b"\xE7\x57",  # LATIN CAPITAL LETTER W WITH DOT ABOVE
    u"\u1E87": b"\xE7\x77",  # LATIN SMALL LETTER W WITH DOT ABOVE
    u"\u1E88": b"\xF2\x57",  # LATIN CAPITAL LETTER W WITH DOT BELOW
    u"\u1E89": b"\xF2\x77",  # LATIN SMALL LETTER W WITH DOT BELOW
    u"\u1E8A": b"\xE7\x58",  # LATIN CAPITAL LETTER X WITH DOT ABOVE
    u"\u1E8B": b"\xE7\x78",  # LATIN SMALL LETTER X WITH DOT ABOVE
    u"\u1E8C": b"\xE8\x58",  # LATIN CAPITAL LETTER X WITH DIAERESIS
    u"\u1E8D": b"\xE8\x78",  # LATIN SMALL LETTER X WITH DIAERESIS
    u"\u1E8E": b"\xE7\x59",  # LATIN CAPITAL LETTER Y WITH DOT ABOVE
    u"\u1E8F": b"\xE7\x79",  # LATIN SMALL LETTER Y WITH DOT ABOVE
    u"\u1E90": b"\xE3\x5A",  # LATIN CAPITAL LETTER Z WITH CIRCUMFLEX
    u"\u1E91": b"\xE3\x7A",  # LATIN SMALL LETTER Z WITH CIRCUMFLEX
    u"\u1E92": b"\xF2\x5A",  # LATIN CAPITAL LETTER Z WITH DOT BELOW
    u"\u1E93": b"\xF2\x7A",  # LATIN SMALL LETTER Z WITH DOT BELOW
    u"\u1E97": b"\xE8\x74",  # LATIN SMALL LETTER T WITH DIAERESIS
    u"\u1E98": b"\xEA\x77",  # LATIN SMALL LETTER W WITH RING ABOVE
    u"\u1E99": b"\xEA\x79",  # LATIN SMALL LETTER Y WITH RING ABOVE
    u"\u1EA0": b"\xF2\x41",  # LATIN CAPITAL LETTER A WITH DOT BELOW
    u"\u1EA1": b"\xF2\x61",  # LATIN SMALL LETTER A WITH DOT BELOW
    u"\u1EA2": b"\xE0\x41",  # LATIN CAPITAL LETTER A WITH HOOK ABOVE
    u"\u1EA3": b"\xE0\x61",  # LATIN SMALL LETTER A WITH HOOK ABOVE
    u"\u1EA4": b"\xE2\xE3\x41",  # LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND ACUTE
    u"\u1EA5": b"\xE2\xE3\x61",  # LATIN SMALL LETTER A WITH CIRCUMFLEX AND ACUTE
    u"\u1EA6": b"\xE1\xE3\x41",  # LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND GRAVE
    u"\u1EA7": b"\xE1\xE3\x61",  # LATIN SMALL LETTER A WITH CIRCUMFLEX AND GRAVE
    u"\u1EA8": b"\xE0\xE3\x41",  # LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE
    u"\u1EA9": b"\xE0\xE3\x61",  # LATIN SMALL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE
    u"\u1EAA": b"\xE4\xE3\x41",  # LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND TILDE
    u"\u1EAB": b"\xE4\xE3\x61",  # LATIN SMALL LETTER A WITH CIRCUMFLEX AND TILDE
    u"\u1EAC": b"\xE3\xF2\x41",  # LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND DOT BELOW
    u"\u1EAD": b"\xE3\xF2\x61",  # LATIN SMALL LETTER A WITH CIRCUMFLEX AND DOT BELOW
    u"\u1EAE": b"\xE2\xE6\x41",  # LATIN CAPITAL LETTER A WITH BREVE AND ACUTE
    u"\u1EAF": b"\xE2\xE6\x61",  # LATIN SMALL LETTER A WITH BREVE AND ACUTE
    u"\u1EB0": b"\xE1\xE6\x41",  # LATIN CAPITAL LETTER A WITH BREVE AND GRAVE
    u"\u1EB1": b"\xE1\xE6\x61",  # LATIN SMALL LETTER A WITH BREVE AND GRAVE
    u"\u1EB2": b"\xE0\xE6\x41",  # LATIN CAPITAL LETTER A WITH BREVE AND HOOK ABOVE
    u"\u1EB3": b"\xE0\xE6\x61",  # LATIN SMALL LETTER A WITH BREVE AND HOOK ABOVE
    u"\u1EB4": b"\xE4\xE6\x41",  # LATIN CAPITAL LETTER A WITH BREVE AND TILDE
    u"\u1EB5": b"\xE4\xE6\x61",  # LATIN SMALL LETTER A WITH BREVE AND TILDE
    u"\u1EB6": b"\xE6\xF2\x41",  # LATIN CAPITAL LETTER A WITH BREVE AND DOT BELOW
    u"\u1EB7": b"\xE6\xF2\x61",  # LATIN SMALL LETTER A WITH BREVE AND DOT BELOW
    u"\u1EB8": b"\xF2\x45",  # LATIN CAPITAL LETTER E WITH DOT BELOW
    u"\u1EB9": b"\xF2\x65",  # LATIN SMALL LETTER E WITH DOT BELOW
    u"\u1EBA": b"\xE0\x45",  # LATIN CAPITAL LETTER E WITH HOOK ABOVE
    u"\u1EBB": b"\xE0\x65",  # LATIN SMALL LETTER E WITH HOOK ABOVE
    u"\u1EBC": b"\xE4\x45",  # LATIN CAPITAL LETTER E WITH TILDE
    u"\u1EBD": b"\xE4\x65",  # LATIN SMALL LETTER E WITH TILDE
    u"\u1EBE": b"\xE2\xE3\x45",  # LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND ACUTE
    u"\u1EBF": b"\xE2\xE3\x65",  # LATIN SMALL LETTER E WITH CIRCUMFLEX AND ACUTE
    u"\u1EC0": b"\xE1\xE3\x45",  # LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND GRAVE
    u"\u1EC1": b"\xE1\xE3\x65",  # LATIN SMALL LETTER E WITH CIRCUMFLEX AND GRAVE
    u"\u1EC2": b"\xE0\xE3\x45",  # LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE
    u"\u1EC3": b"\xE0\xE3\x65",  # LATIN SMALL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE
    u"\u1EC4": b"\xE4\xE3\x45",  # LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND TILDE
    u"\u1EC5": b"\xE4\xE3\x65",  # LATIN SMALL LETTER E WITH CIRCUMFLEX AND TILDE
    u"\u1EC6": b"\xE3\xF2\x45",  # LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND DOT BELOW
    u"\u1EC7": b"\xE3\xF2\x65",  # LATIN SMALL LETTER E WITH CIRCUMFLEX AND DOT BELOW
    u"\u1EC8": b"\xE0\x49",  # LATIN CAPITAL LETTER I WITH HOOK ABOVE
    u"\u1EC9": b"\xE0\x69",  # LATIN SMALL LETTER I WITH HOOK ABOVE
    u"\u1ECA": b"\xF2\x49",  # LATIN CAPITAL LETTER I WITH DOT BELOW
    u"\u1ECB": b"\xF2\x69",  # LATIN SMALL LETTER I WITH DOT BELOW
    u"\u1ECC": b"\xF2\x4F",  # LATIN CAPITAL LETTER O WITH DOT BELOW
    u"\u1ECD": b"\xF2\x6F",  # LATIN SMALL LETTER O WITH DOT BELOW
    u"\u1ECE": b"\xE0\x4F",  # LATIN CAPITAL LETTER O WITH HOOK ABOVE
    u"\u1ECF": b"\xE0\x6F",  # LATIN SMALL LETTER O WITH HOOK ABOVE
    u"\u1ED0": b"\xE2\xE3\x4F",  # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND ACUTE
    u"\u1ED1": b"\xE2\xE3\x6F",  # LATIN SMALL LETTER O WITH CIRCUMFLEX AND ACUTE
    u"\u1ED2": b"\xE1\xE3\x4F",  # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND GRAVE
    u"\u1ED3": b"\xE1\xE3\x6F",  # LATIN SMALL LETTER O WITH CIRCUMFLEX AND GRAVE
    u"\u1ED4": b"\xE0\xE3\x4F",  # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE
    u"\u1ED5": b"\xE0\xE3\x6F",  # LATIN SMALL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE
    u"\u1ED6": b"\xE4\xE3\x4F",  # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND TILDE
    u"\u1ED7": b"\xE4\xE3\x6F",  # LATIN SMALL LETTER O WITH CIRCUMFLEX AND TILDE
    u"\u1ED8": b"\xE3\xF2\x4F",  # LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND DOT BELOW
    u"\u1ED9": b"\xE3\xF2\x6F",  # LATIN SMALL LETTER O WITH CIRCUMFLEX AND DOT BELOW
    u"\u1EE4": b"\xF2\x55",  # LATIN CAPITAL LETTER U WITH DOT BELOW
    u"\u1EE5": b"\xF2\x75",  # LATIN SMALL LETTER U WITH DOT BELOW
    u"\u1EE6": b"\xE0\x55",  # LATIN CAPITAL LETTER U WITH HOOK ABOVE
    u"\u1EE7": b"\xE0\x75",  # LATIN SMALL LETTER U WITH HOOK ABOVE
    u"\u1EF2": b"\xE1\x59",  # LATIN CAPITAL LETTER Y WITH GRAVE
    u"\u1EF3": b"\xE1\x79",  # LATIN SMALL LETTER Y WITH GRAVE
    u"\u1EF4": b"\xF2\x59",  # LATIN CAPITAL LETTER Y WITH DOT BELOW
    u"\u1EF5": b"\xF2\x79",  # LATIN SMALL LETTER Y WITH DOT BELOW
    u"\u1EF6": b"\xE0\x59",  # LATIN CAPITAL LETTER Y WITH HOOK ABOVE
    u"\u1EF7": b"\xE0\x79",  # LATIN SMALL LETTER Y WITH HOOK ABOVE
    u"\u1EF8": b"\xE4\x59",  # LATIN CAPITAL LETTER Y WITH TILDE
    u"\u1EF9": b"\xE4\x79",  # LATIN SMALL LETTER Y WITH TILDE
    u"\u1FEF": b"\x60",  # GREEK VARIA
    u"\u2113": b"\xC1",  # SCRIPT SMALL L
    u"\u2117": b"\xC2",  # SOUND RECORDING COPYRIGHT
    u"\u212A": b"\x4B",  # KELVIN SIGN
    u"\u212B": b"\xEA\x41",  # ANGSTROM SIGN
    u"\u266D": b"\xA9",  # MUSIC FLAT SIGN
    u"\u266F": b"\xC4",  # MUSIC SHARP SIGN
}

UNICODE_TO_ANSEL_MODIFIERS = {
    u"\u0300": b"\xE1",  # COMBINING GRAVE ACCENT
    u"\u0301": b"\xE2",  # COMBINING ACUTE ACCENT
    u"\u0302": b"\xE3",  # COMBINING CIRCUMFLEX ACCENT
    u"\u0303": b"\xE4",  # COMBINING TILDE
    u"\u0304": b"\xE5",  # COMBINING MACRON
    u"\u0306": b"\xE6",  # COMBINING BREVE
    u"\u0307": b"\xE7",  # COMBINING DOT ABOVE
    u"\u0308": b"\xE8",  # COMBINING DIAERESIS
    u"\u0309": b"\xE0",  # COMBINING HOOK ABOVE
    u"\u030A": b"\xEA",  # COMBINING RING ABOVE
    u"\u030B": b"\xEE",  # COMBINING DOUBLE ACUTE ACCENT
    u"\u030C": b"\xE9",  # COMBINING CARON
    u"\u0310": b"\xEF",  # COMBINING CANDRABINDU
    u"\u0313": b"\xFE",  # COMBINING COMMA ABOVE
    u"\u0315": b"\xED",  # COMBINING COMMA ABOVE RIGHT
    u"\u031C": b"\xF8",  # COMBINING LEFT HALF RING BELOW
    u"\u0323": b"\xF2",  # COMBINING DOT BELOW
    u"\u0324": b"\xF3",  # COMBINING DIAERESIS BELOW
    u"\u0325": b"\xF4",  # COMBINING RING BELOW
    u"\u0326": b"\xF7",  # COMBINING COMMA BELOW
    u"\u0327": b"\xF0",  # COMBINING CEDILLA
    u"\u0328": b"\xF1",  # COMBINING OGONEK
    u"\u032E": b"\xF9",  # COMBINING BREVE BELOW
    u"\u0332": b"\xF6",  # COMBINING LOW LINE
    u"\u0333": b"\xF5",  # COMBINING DOUBLE LOW LINE
    u"\u0340": b"\xE1",  # COMBINING GRAVE TONE MARK
    u"\u0341": b"\xE2",  # COMBINING ACUTE TONE MARK
    u"\u0343": b"\xFE",  # COMBINING GREEK KORONIS
    u"\u0344": b"\xE2\xE8",  # COMBINING GREEK DIALYTIKA TONOS
    u"\uFE20": b"\xEB",  # COMBINING LIGATURE LEFT HALF
    u"\uFE21": b"\xEC",  # COMBINING LIGATURE RIGHT HALF
    u"\uFE22": b"\xFA",  # COMBINING DOUBLE TILDE LEFT HALF
    u"\uFE23": b"\xFB",  # COMBINING DOUBLE TILDE RIGHT HALF
}


class Codec(codec.Codec):
    name = "ansel"
    encode_char_map = UNICODE_TO_ANSEL
    encode_modifier_map = UNICODE_TO_ANSEL_MODIFIERS
    decode_char_map = ANSEL_TO_UNICODE
    decode_modifier_map = ANSEL_TO_UNICODE_MODIFIERS


class IncrementalDecoder(incremental.IncrementalDecoder):
    name = "ansel"
    decode_char_map = ANSEL_TO_UNICODE
    decode_modifier_map = ANSEL_TO_UNICODE_MODIFIERS


class IncrementalEncoder(incremental.IncrementalEncoder):
    name = "ansel"
    encode_char_map = UNICODE_TO_ANSEL
    encode_modifier_map = UNICODE_TO_ANSEL_MODIFIERS


class StreamReader(Codec, codecs.StreamReader):
    pass


class StreamWriter(Codec, codecs.StreamWriter):
    pass


def getregentry():
    return codecs.CodecInfo(
        name="ansel",
        encode=Codec().encode,
        decode=Codec().decode,
        incrementalencoder=IncrementalEncoder,
        incrementaldecoder=IncrementalDecoder,
        streamreader=StreamReader,
        streamwriter=StreamWriter,
    )
